@@ -6,14 +6,25 @@ setClass("claytonCopula",
 
 genFunClayton <- function(copula, u) {
   alpha <- copula@parameters[1]
-  u^(-alpha) - 1
+  (u^(-alpha) - 1) / alpha
 }
 
 genInvClayton <- function(copula, s) {
   alpha <- copula@parameters[1]
-  (1 + s)^(-1/alpha)
+  (1 + alpha * s)^(-1/alpha) 
 }
 
+genFunDer1Clayton <- function(copula, u) {
+  alpha <- copula@parameters[1]
+  eval(claytonCopula.genfun.expr[1])
+}
+
+genFunDer2Clayton <- function(copula, u) {
+  alpha <- copula@parameters[1]
+  eval(claytonCopula.genfun.expr[2])
+}
+
+
 claytonCopula <- function(param, dim = 2) {
   ## get expressions of cdf and pdf
   cdfExpr <- function(n) {
@@ -44,7 +55,7 @@ claytonCopula <- function(param, dim = 2) {
              parameters = param[1],
              exprdist = c(cdf = cdf, pdf = pdf),
              param.names = "param",
-             param.lowbnd = -1,
+             param.lowbnd = ifelse(dim == 2, -1, 0),
              param.upbnd = Inf,
              message = "Clayton copula family; Archimedean copula")
   val
@@ -76,9 +87,8 @@ pclaytonCopula <- function(copula, u) {
   dim <- copula@dimension
   if (is.vector(u)) u <- matrix(u, ncol = dim)
   alpha <- copula@parameters[1]
-  if (abs(alpha) <= 100 * .Machine$double.eps) return (apply(u, 1, prod))
+  if (abs(alpha) <= .Machine$double.eps^.9) return (apply(u, 1, prod))
   cdf <- copula@exprdist$cdf
-  dim <- copula@dimension
   for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
   val <- eval(cdf)
   pmax(val, 0)
@@ -89,18 +99,36 @@ dclaytonCopula <- function(copula, u) {
   dim <- copula@dimension
   if (is.vector(u)) u <- matrix(u, ncol = dim)
   alpha <- copula@parameters[1]
-  if (abs(alpha) <= 100 * .Machine$double.eps) return (rep(1, length(u)))
+  if (abs(alpha) <= .Machine$double.eps^.9) return (rep(1, nrow(u)))
   pdf <- copula@exprdist$pdf
-  dim <- copula@dimension
   for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
   val <- eval(pdf)
-  val[apply(u, 1, function(v) any(v <= 0))] <- 0
-  val[apply(u, 1, function(v) any(v >= 1))] <- 0
-  if (alpha < 0) {
-    cdf <- pcopula(copula, u)
-    bad <- cdf == 0
-    val[bad] <- 0
-  }
+  val[apply(u, 1, function(v) any(v < 0))] <- 0
+  val[apply(u, 1, function(v) any(v > 1))] <- 0
+##   if (alpha < 0) {
+##     cdf <- pcopula(copula, u)
+##     bad <- cdf == 0
+##     val[bad] <- 0
+##   }
+  val
+}
+
+dclaytonCopula.pdf <- function(copula, u) {
+  dim <- copula@dimension
+  if (dim > 10) stop("Clayton copula PDF not implemented for dimension > 10.")
+  if (is.vector(u)) u <- matrix(u, nrow = 1)
+  for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
+  alpha <- copula@parameters[1]
+  if (abs(alpha) <= .Machine$double.eps^.9) return (rep(1, nrow(u)))
+  val <- eval(claytonCopula.pdf.algr[dim])
+  ## clean up
+  val[apply(u, 1, function(v) any(v < 0))] <- 0
+  val[apply(u, 1, function(v) any(v > 1))] <- 0
+  
+  val[apply(u, 1, function(v) any(v == 0) & any(v > 0))] <- 0
+  
+  ## if (alpha > 0)
+  ## else 
   val
 }
 
@@ -110,11 +138,30 @@ kendallsTauClaytonCopula <- function(copula) {
   alpha / (alpha + 2)
 }
 
+calibKendallsTauClaytonCopula <- function(copula, tau) {
+  2 * tau / (1 - tau)
+}
+
+tailIndexClaytonCopula <- function(copula, ...) {
+  upper <- 0
+  alpha <- copula@parameters
+  lower <- ifelse(alpha > 0, 2 ^ (-1/alpha), 0)
+  c(lower=lower, upper=upper)
+}
+
+
 setMethod("rcopula", signature("claytonCopula"), rclaytonCopula)
 setMethod("pcopula", signature("claytonCopula"), pclaytonCopula)
 setMethod("dcopula", signature("claytonCopula"), dclaytonCopula)
 
 setMethod("genFun", signature("claytonCopula"), genFunClayton)
 setMethod("genInv", signature("claytonCopula"), genInvClayton)
+setMethod("genFunDer1", signature("claytonCopula"), genFunDer1Clayton)
+setMethod("genFunDer2", signature("claytonCopula"), genFunDer2Clayton)
 
 setMethod("kendallsTau", signature("claytonCopula"), kendallsTauClaytonCopula)
+#setMethod("spearmansRho", signature("claytonCopula"), spearmansRhoClaytonCopula)
+setMethod("tailIndex", signature("claytonCopula"), tailIndexClaytonCopula)
+
+setMethod("calibKendallsTau", signature("claytonCopula"), calibKendallsTauClaytonCopula)
+
